function pkglist(refreshlist)
%PKGLIST List available octave-forge packages
%
%   PKGLIST Lists available packages from the file packages.mat, if found.
%   If packages.mat is not found, PKGLIST(1) is executed.
%  
%   PKGLIST(refreshlist) refreshes list by downloading packages.html if
%   refreshlist = 1.
%
%   See also PKGGET.

% Note: The "right" way to do all of this is to store the Name, Description,
% URL strings in an XML file (say, info.xml) or a JavaScript Object in a
% javascript file, (say info.js).  Both this Octave program and
% packages.html would then just need to read the info file.

% It would also be helpful if a MD5 was listed for each file.  
  
% Could test if refresh needed by comparing timestamp of web page and
% comparing with that from last download.  Can the curl library return only
% the HTTP header, which should contain this information?

if (nargin < 1)  
  refreshlist = 0; 
end

packagehtml = 'packages.html';
packagesmat = 'packages.mat';
baseurl = 'http://octave.dbateman.org/packages';
domain = 'http://octave.dbateman.org';

if (refreshlist==1) || ~exist(packagesmat)

  url = sprintf('%s/%s',domain,packagehtml);
  [F,F_ok] = xurlwrite(url,packagehtml);
  
  % Read entire file, preserving newlines using %c instead of %s
  fid = fopen(packagehtml,'rt');
  F = fscanf(fid,'%c');
  fclose(fid);

  key1 = sprintf('_details'); 
  [I,J1] = regexp(F,key1);
  J1 = J1+1;
  
  key2 = sprintf('javascript:fold');
  [I,J2] = regexp(F,key2);
  J2 = J2+1;
  
  key3 = sprintf('package_link" href=');
  [I,J3] = regexp(F,key3);
  J3 = J3+1;
  J3 = J3([1:2:length(J3)]); % Remove double occurences
  
  for i = 1:length(J1)
    I = findstr(F(J1(i):J1(i)+500),'</p>'); % Assume description < 500 chars
    Desc{i} = regexprep(F(J1(i)+2:J1(i)+I(1)-4),'\n','');
    
    I = findstr(F(J2(i):J2(i)+80),';');    % Assume name < 80 chars
    Name{i} = F(J2(i)+2:J2(i)+I(1)-4); 
    
    I = findstr(F(J3(i):J3(i)+100),'gz'); % Assume package name < 100 chars
    Link{i} = F(J3(i)+1:J3(i)+I(1));
    eval(sprintf('save %s Link Desc Name',packagesmat));
  end  

  pkglist(0);
  
else

   if exist(packagesmat)
     eval(sprintf('load %s',packagesmat));
   else
     if ~exist(packagesmat)     
       pkglist(1);
     end
     if ~exist(packagesmat)
       error('pkglist: packages.mat not found and PKGLIST(1) failed.')
     end
   end
   
   for i = 1:length(Name)
     spaces = blanks(20-length(Name{i}));
     ld = min(55,length(Desc{i}));
     if (ld == 55)
       fprintf('%s%s%s ...\n',Name{i},spaces,Desc{i}(1:ld));
     else
       fprintf('%s%s%s\n',Name{i},spaces,Desc{i});
     end
   end
   
end


